/***************************************************************************
                          main.c  -  WM Applet for Compaq Armada E500
                             -------------------
    begin                : Wed Jun 22 2005
    copyright            : (C) 2005 by Andrei Gavrila
    email                : andreig@revolutionstudio.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/Xutil.h>

#include <X11/extensions/shape.h>

#include "imageb.h"
#include "image0.h"
#include "image1.h"
#include "image2.h"
#include "image3.h"
#include "image4.h"
#include "image5.h"
#include "image6.h"
#include "image7.h"

#include "lcd.h"

#define CLICKL 0
#define CLICKR 1

static int brightness;

/*
 * Process the Expose event and call the function to set the LCD Brightness
 *
 */

int processExpose(Display *dpy, GC gc, Pixmap * XPM, Window win1, Window win2)
{
  brightness = lcdSetBrightness(brightness);
  brightness = brightness > 0 ? brightness : 0;

  XCopyArea(dpy, XPM[brightness], win1, gc, 0, 0, 64, 64, 0, 0);
  XCopyArea(dpy, XPM[brightness], win2, gc, 0, 0, 64, 64, 0, 0);

  return 0;
}

/*
 * Process the Click Event (CLICKL and CLICKR). Return 1 on change, 0 otherwise
 *
 */

int processClick(int type)
{
  int b = brightness;

  switch (type) {
    case CLICKL :
      brightness += brightness < 7 ? 1 : 0;
    break;

    case CLICKR :
      brightness -= brightness > 0 ? 1 : 0;
    break;
  }

  if (b != brightness)
    return 1;
  else
    return 0;
}

/*
 * Application entry point
 *
 */

int main(int argc, char** argv)
{
  Display      *dpy;
  int           screen;
  Window        winRoot, winMain, winIcon;
  GC            gc;
  unsigned long pixelBlack, pixelWhite;
  XSizeHints    sizeHints;
  XWMHints     *wmHints;  
  XClassHint    classHint;
  XEvent        ev;
  Pixmap        pixmask; 
  Pixmap        XPM[8];

  dpy     = XOpenDisplay(NULL);
  screen  = DefaultScreen(dpy);
  winRoot = RootWindow(dpy, screen);

  pixelWhite = WhitePixel(dpy, screen);
  pixelBlack = BlackPixel(dpy, screen);

  sizeHints.x      = 0;
  sizeHints.y      = 0;
  sizeHints.width  = 64;
  sizeHints.height = 64;

  winMain = XCreateSimpleWindow(dpy, winRoot, sizeHints.x, sizeHints.y, sizeHints.width, sizeHints.height, 1, pixelBlack, pixelWhite);
  winIcon = XCreateSimpleWindow(dpy, winRoot, sizeHints.x, sizeHints.y, sizeHints.width, sizeHints.height, 1, pixelBlack, pixelWhite);

  sizeHints.flags = USSize | USPosition;
  XSetWMNormalHints(dpy, winMain, &sizeHints);

  wmHints = XAllocWMHints();

  wmHints->flags         = IconWindowHint | WindowGroupHint;
  wmHints->flags        |= StateHint;
  wmHints->initial_state = WithdrawnState;
  wmHints->window_group  = winMain;
  wmHints->icon_window   = winIcon;

  XSetWMHints(dpy, winMain, wmHints);

  XFree(wmHints);

  classHint.res_name  = argv[0];
  classHint.res_class = "DockApp";

  XSetClassHint(dpy, winMain, &classHint);

  memset(imageb_bits, 0xff, sizeof(imageb_bits));
  pixmask = XCreateBitmapFromData(dpy, winMain, imageb_bits, 64, 64);

  XShapeCombineMask(dpy, winMain, ShapeBounding, 0, 0, pixmask, ShapeSet);
  XShapeCombineMask(dpy, winIcon, ShapeBounding, 0, 0, pixmask, ShapeSet);

  XpmCreatePixmapFromData(dpy, winRoot, image0_xpm, &XPM[0], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image1_xpm, &XPM[1], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image2_xpm, &XPM[2], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image3_xpm, &XPM[3], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image4_xpm, &XPM[4], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image5_xpm, &XPM[5], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image6_xpm, &XPM[6], NULL, NULL);
  XpmCreatePixmapFromData(dpy, winRoot, image7_xpm, &XPM[7], NULL, NULL);

  XSelectInput(dpy, winMain, ExposureMask | ButtonPressMask);
  XSelectInput(dpy, winIcon, ExposureMask | ButtonPressMask);

  XSetCommand(dpy, winMain, argv, argc);

  gc = DefaultGC(dpy, DefaultScreen(dpy));

  XMapWindow(dpy,winMain);

  brightness = lcdGetBrightness();
  brightness = brightness > 0 ? brightness : 0;

  while (1) {
    XNextEvent(dpy, &ev);

    switch (ev.type) {
      case Expose:
        processExpose(dpy, gc, XPM, winMain, winIcon);
      break;

      case ButtonPress:
        switch (ev.xbutton.button) {
          case 1:
            if (processClick(CLICKL))
              processExpose(dpy, gc, XPM, winMain, winIcon);
          break;

          case 3:
            if (processClick(CLICKR))
              processExpose(dpy, gc, XPM, winMain, winIcon);
          break;
        }
      break;
    }
  }

  return 0;
}
